<?php
/*
 画像添付メールのWeb掲示
*/

mb_internal_encoding("UTF-8");
chdir(dirname(__FILE__));

//共通モジュール
set_include_path(get_include_path() . PATH_SEPARATOR . "./PEAR");
require_once './PEAR/Mail_mimeDecode/mimeDecode.php';
require_once './log.php';

//ログ設定（デバッグ用）
log::setfile("./receive.log");
log::setheader(getmypid());
log::setphperror(E_ALL ^ E_NOTICE);

//処理開始
log::put("Start");

//メールソースを標準入力から読み込み
while(!feof(STDIN)) {
	$source .= fread(STDIN, 4096);
}
if (!$source)
	exit;

//
//メールソースの解析
//
$decoder = new Mail_mimeDecode($source);
$params['include_bodies'] = true;
$params['decode_bodies']  = true;
$params['decode_headers'] = true;
$structure = $decoder->decode($params);

//メールヘッダの取得
$mail = getHeader($structure);

//メール本文の取得
$mail += getBody($structure);
//log::put("structure ".print_r($structure, true));


//メールを掲示するディレクトリ（環境に応じて変更してください。）
$dir = "./temp/".uniqid("m", true);
mkdir($dir);
chmod($dir, 0755);
log::put("dir ".$dir);
//メールソースのバックアップ
file_put_contents($dir."/mail.txt", $source);
chmod($dir."/mail.txt", 0604);


//
//メール掲示
//
makehtml($mail, $dir);
log::put("mail ".print_r($mail, true));


//
//メール返信
//
replymail($mail, $dir);


//終了
log::put("End");

exit;


/*
 機能 メール掲載（HTML組立）
 引数 $mail メール解析情報
      $dir 掲載先ディレクトリ
 戻値 メール送信結果
*/
function makehtml(&$mail, $dir) {
	//メール掲示用HTML作成
	foreach ($mail['text'] as $val) {
		switch ($val['type']) {
		case "plain":
			//プレインテキスト形式用のHTMLテンプレート
			$html = '<html><head><META http-equiv="Content-Type" content="text/html; charset=Shift_JIS"><title>{title}</title></head><body><div>{title}</div><hr><div>{content}</div></body></html>';
			$html = str_replace("{content}", nl2br(htmlspecialchars($val['body'])), $html);
			break;

		case "html":
			//HTML形式（デコメール等）
			$html = $val['body'];
	
			//文字セットの変更
			$pattern = '/<(meta)(.*?)charset=[\w\-]+(.*?)>/i';
			$replacement = "<$1$2charset=Shift_JIS$3>";
			$html = preg_replace($pattern, $replacement, $html);
			
			//タイトル要素のチェック
			if (preg_match('/<head>(.*?)<title>(.*?)<\/title>(.*?)<\/head>/is', $html, $matches)) {
				if (trim($matches[2])=="") {
					//タイトルの置換文字を設定
					$pattern = '/(<title>)(.*?)(<\/title>)/is';
					$replacement = "$1{title}$3";
					$html = preg_replace($pattern, $replacement, $html);
				}
			} else {
				//タイトル要素の追加
				$pattern = '/(<head>)(.*?)(<\/head>)/is';
				$replacement = "$1$2<title>{title}</title>$3";
				$html = preg_replace($pattern, $replacement, $html);
			}
	
			break 2; //プレインテキスト形式より優先する
		}
	}
	
	//タイトルの設定
	$html = str_replace("{title}", htmlspecialchars($mail['subject']), $html);

	//添付画像の出力
	$images = "";
	if (isset($mail['image'])) {
		foreach ($mail['image'] as $key=>$val) {
			$file = "p".$key.strrchr($val['name'], ".");
			//ファイル保存
			file_put_contents($dir."/".$file, $val['body']);
			chmod($dir."/".$file, 0604);
			//HTML
			if ($val['cid']) {
				//インライン画像
				$pattern = '/(<img.*?src="?)cid:'.preg_quote($val['cid'], '/').'("?.*?>)/is';
				$replacement = "$1".$file."$2";
				$html = preg_replace($pattern, $replacement, $html);
			} else {
				//添付画像(attachment)
				$images .= '<img src="'.$file.'" alt="'.$val['name'].'"><br>';
			}
			//画像データの解放
			unset($mail['image'][$key]['body']);
		}
	}

	//添付画像の追加
	if ($images) {
		//画像要素の追加
		$pattern = '/(<body>)(.*?)(<\/body>)/is';
		$replacement = "$1$2<hr><div>".$images."<div>$3";
		$html = preg_replace($pattern, $replacement, $html);
	}

	//HTML出力
	file_put_contents($dir."/index.html", mb_convert_encoding($html, "SJIS", mb_internal_encoding()));
	chmod($dir."/index.html", 0604); //WWWサーバが参照できるパーミッションを設定する

}


/*
 機能 メール返信
 引数 $mail メール解析情報
      $dir 掲載先ディレクトリ
 戻値 メール送信結果
*/
function replymail($mail, $dir) {
	$to = $mail['email'];
	$subject = "Re: ".$mail['subject'];
	$body = "メールを掲示しました。\n";
	$body .= "http://yourdomain/picturemail/temp/".basename($dir)."/\n";
	$body .= "※なお掲示はしばらくして削除されます。\n";
	//メールヘッダ
	$headers = "From: p@yourdomain\n"; //送信者
	$headers .= setreplyheader($mail);
	$headers .= "X-Mailer: PHP/".phpversion()."\n"; //メール送信プログラム
	//エラー返送先
	$returnpath = "-f"."info@yourdomain";
	//メール送信
	return mb_send_mail($to, $subject, $body, $headers, $returnpath);
}
/*
 機能 メールヘッダの解析
 引数 $structure PEARのmime_decodeの構造
 戻値 ヘッダの連想配列
*/
function getHeader($structure) {
	$mail = array();
	//メールヘッダ情報の取得
	$mail['returnpath'] = trim($structure->headers['return-path'], '<>');
	$mail['subject'] = mb_convert_encoding(mb_decode_mimeheader($structure->headers['subject']), mb_internal_encoding(), "auto");
	$mail['date'] = date("Y-m-d H:i:s", strtotime($structure->headers['date']));
	$mail['from'] = mb_convert_encoding(mb_decode_mimeheader($structure->headers['from']), mb_internal_encoding(), "auto");
	
	$pos = strrpos($mail['from'], ' ');
	if ($pos) {
		$mail['sender'] = trim(substr($mail['from'], 0, $pos), '"');
		$mail['email'] = trim(substr($mail['from'], $pos+1), '<>');
	} else {
		$mail['sender'] = "";
		$mail['email'] = trim($mail['from'], '<>');
	}

	//リファレンス
	$mail['message-id'] = $structure->headers['message-id'];
	$mail['references'] = $structure->headers['references'];

	return $mail;
}

/*
 機能 メール本文の解析
 引数 $structure PEARのmime_decodeの構造
      $body 指定不要（関数内部のみで利用）
 戻値 本文テキスト、添付画像の連想配列
*/

function getBody($structure, &$body=array()) {

	switch (strtolower($structure->ctype_primary)) {
	case "text": //テキスト本文のみのメール
		$body['text'][] = array(
			"type" => $structure->ctype_secondary,
			"body" => mb_convert_encoding($structure->body, mb_internal_encoding(), "auto")
		);
		break;

	case "multipart": //複数本文があるメール
		//本文を１件づつ処理する
		foreach ($structure->parts as $part) {

			switch (strtolower($part->ctype_primary)) {
			case "image": //画像ファイル部分
				$body['image'][] = array(
					'type' => $part->ctype_secondary,
					'name' => $part->ctype_parameters['name'],
					'cid' => trim($part->headers['content-id'], '<>'),
					'body' => $part->body //画像データ
				);
				break;
			case "text": //テキスト本文部分
				$body['text'][] = array(
					'type' => $part->ctype_secondary,
					'body' => mb_convert_encoding($part->body, mb_internal_encoding(), "auto")
				);
				break;
			case "multipart": //マルチパートの中にマルチパートがある場合
				//MS-OutlookExpressからHTML形式で送信した場合）
				getbody($part, $body);
				break;
			}

		}

	}

	return $body;
}

/*
 機能 返信用メールヘッダの組立
 引数 $mail 受信メールヘッダ（連想配列）
 戻値 返信用メールヘッダ
*/
function setreplyheader($mail) {
	$headers = "In-Reply-To: ".$mail['message-id']."\n"; //返信先
	$headers .= "References: ".trim($mail['references']." ".$mail['message-id'])."\n"; //リファレンス
	return $headers;
}

?>